//
//	wsock.h
//
//	Created on: May 18, 2022
//      Author: vagner
//

#pragma once

#include <string>

#include <vCommon.h>
#include <LogItem.h>

namespace vsock
{

    class wsock : LogItem
    {
    public:

        //  ctor
        //  name - логическое имя
        //  rto - таймаут приема (msec)
        //  sto - таймаут передачи (msec)
        wsock ( std::string &name, const std::string &Host, const int Port, int rOut, int sOut );

        //  dtor
        virtual ~wsock ( );

        //  открыть соединение
        bool Open ( );

        //  закрыть соединение
        void Close ( );

        //  принять
        //  возвращает:
        //  <0  ошибка
        //   0  разрыв связи
        //  >0  размер принятого
        int Recv ( void *buf, int bSize );

        //  принять указанный размер
        bool RecvAll ( void *buf, int bSize );

        //  отправить
        //  возвращает:
        //  <0  ошибка
        //  >0  размер переданного
        int Send ( void *buf, int bSize );

        //  отправить указанный размер
        bool SendAll ( void *buf, int bSize );

        //  наличие соединения
        inline bool IsOpen ( )
        {
            return _IsOpen;
        }

        //  валидность сокета
        inline bool IsValidSocket ( )
        {
            return fd > 0;
        }

        //  сбросить приемный буфер
        void flush_input ( );

        //  set_timeout
        void set_timeout ( int rto, int sto );

        //  restart
        bool restart ( );

    private:

        //  сокет
        int fd;

        //  соединение установлено
        bool _IsOpen;

        //  таймаут приема
        int rto;

        //  таймаут передачи
        int sto;

        //  host
        std::string Host;

        //  port
        int Port;

        //  опрос
        bool poll ( int rto );

    };

} /* namespace vsock */
